<?php

namespace Drupal\form_mode_control_role_assignment\EventSubscriber;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Routing\RouteMatchInterface;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Drupal\user\Event\UserEvents;
use Drupal\user\Event\UserEvent;

class UserRegistrationRoleSubscriber implements EventSubscriberInterface {
  
  protected $configFactory;
  protected $routeMatch;
  protected $entityTypeManager;

  public function __construct(
    ConfigFactoryInterface $config_factory, 
    RouteMatchInterface $route_match,
    EntityTypeManagerInterface $entity_type_manager
  ) {
    $this->configFactory = $config_factory;
    $this->routeMatch = $route_match;
    $this->entityTypeManager = $entity_type_manager;
  }

  public static function getSubscribedEvents() {
    return [
      UserEvents::USER_REGISTER => 'onUserRegister',
    ];
  }

  public function onUserRegister(UserEvent $event) {
    $current_form_mode = $this->routeMatch->getParameter('form_mode');
    
    if ($current_form_mode) {
      $role_mapping = $this->configFactory
        ->get('form_mode_control_role_assignment.settings')
        ->get('role_mappings');
      
      $user = $event->getUser();
      
      if (isset($role_mapping[$current_form_mode])) {
        $user->addRole($role_mapping[$current_form_mode]);
        $user->save();
      }
    }
  }
}