<?php

namespace Drupal\form_mode_control_role_assignment\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\user\Entity\Role;

class FormModeRoleMappingForm extends ConfigFormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'form_mode_role_mapping_form';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [
      'form_mode_control_role_assignment.settings',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('form_mode_control_role_assignment.settings');
    
    // Get all user form modes
    $form_modes = \Drupal::service('entity_display.repository')
      ->getFormModes('user');
    
    // Get all user roles
    $roles = Role::loadMultiple();
    
    $form['role_mappings'] = [
      '#type' => 'details',
      '#title' => $this->t('Form Mode to Role Mapping'),
      '#open' => TRUE,
    ];
    
    foreach ($form_modes as $form_mode_id => $form_mode) {
      $form['role_mappings']['mapping_' . $form_mode_id] = [
        '#type' => 'select',
        '#title' => $this->t('Role for Form Mode: @form_mode', ['@form_mode' => $form_mode['label']]),
        '#options' => array_combine(
          array_keys($roles),
          array_map(function($role) { return $role->label(); }, $roles)
        ),
        '#default_value' => $config->get('role_mappings.' . $form_mode_id) ?? '',
        '#empty_option' => $this->t('- No Role -'),
      ];
    }
    
    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $config = $this->configFactory->getEditable('form_mode_control_role_assignment.settings');
    
    $form_modes = \Drupal::service('entity_display.repository')
      ->getFormModes('user');
    
    foreach ($form_modes as $form_mode_id => $form_mode) {
      $role = $form_state->getValue('mapping_' . $form_mode_id);
      $config->set('role_mappings.' . $form_mode_id, $role);
    }
    
    $config->save();
    
    parent::submitForm($form, $form_state);
  }
}